# Contains methods for setting up more complicated IHT ModVars; others can just be set up in ICModVars.py

import numpy as np

import SpectrumCommon.Const.GB.GBConst as gbc

import SpectrumCommon.Const.BG.BGConst as bgc
import SpectrumCommon.Const.BG.BGConstLink as bgcl
import SpectrumCommon.Const.BG.BGModVarFields as bgmvf

#######################################################################################################
#                                                                                                     #
#                                       Budgets                                                       #
#                                                                                                     #
#######################################################################################################

def BG_create_budget_rec(ID: str, name: str, string_constant: str, num_dynamic_staff_types: int):

    budget_rec = {
        bgmvf.BG_BUDGET_ID                                           : ID,                                                
        bgmvf.BG_BUDGET_NAME                                         : name,     
        bgmvf.BG_BUDGET_STR_CONST                                    : string_constant,             
        bgmvf.BG_BUDGET_COST_INTERVS_DRUGS_SUPPLIES_IC               : bgc.BG_IC_COST_INTERVENTIONS_MST_ID,
        bgmvf.BG_BUDGET_AGGR_DELIV_CHANS_IC                          : True,
        bgmvf.BG_BUDGET_CUSTOM                                       : False,
        bgmvf.BG_BUDGET_ACTIVE                                       : True,
        bgmvf.BG_BUDGET_FW_ID                                        : '-1',

        bgmvf.BG_COST_CAT_IC_ID                                      : bgc.BG_IC_DRUGS_SUPPLIES_CC_MST_ID,

        bgmvf.BG_BUDGET_COST_LEVEL_PC_ID                             : bgc.BG_COST_COSTING_SECTIONS_LEVEL_MST_ID,
        bgmvf.BG_BUDGET_COST_LEVEL_IS_ID                             : bgc.BG_COST_COSTING_SECTIONS_LEVEL_MST_ID,
        bgmvf.BG_BUDGET_COST_LEVEL_HW_ID                             : bgc.BG_COST_COSTING_SECTIONS_LEVEL_MST_ID,

        bgmvf.BG_BUDGET_AGGR_INSERV_TRAIN_ALL_STAFF_HW               : False,
        bgmvf.BG_BUDGET_AGGR_PRESERV_TRAIN_ALL_STAFF_HW              : False, 
        bgmvf.BG_BUDGET_AGGR_TOTAL_COMPENS_ALL_STAFF_HW              : np.full((bgcl.BG_HW_NUM_STAFFING_LEVELS, bgcl.BG_HW_NUM_COMPENS_TYPES), False).tolist(),
        bgmvf.BG_BUDGET_AGGR_STAFF_INSERV_TRAIN_ALL_STAFF_LEV_HW     : np.full((num_dynamic_staff_types), False).tolist(),
        bgmvf.BG_BUDGET_AGGR_STAFF_LEV_INSERV_TRAIN_ALL_STAFF_HW     : np.full((bgcl.BG_HW_NUM_STAFFING_LEVELS), False).tolist(),
        bgmvf.BG_BUDGET_AGGR_TOTAL_COMPENS_STAFF_LEV_COMPENS_TYPE_HW : np.full((num_dynamic_staff_types), False).tolist(),

        bgmvf.BG_BUDGET_GROUP_IND_AGGR_SET_IS                        : [],
        bgmvf.BG_BUDGET_FACILITY_IND_AGGR_SET_IS                     : [],

        bgmvf.BG_BUDGET_COST_LEVEL_HS_ID                             : bgc.BG_COST_COSTING_SECTIONS_LEVEL_MST_ID,
        bgmvf.BG_BUDGET_COST_LEVEL_GV_ID                             : bgc.BG_COST_COSTING_SECTIONS_LEVEL_MST_ID,
        bgmvf.BG_BUDGET_COST_LEVEL_HF_ID                             : bgc.BG_COST_COSTING_SECTIONS_LEVEL_MST_ID,
    }

    return budget_rec

#######################################################################################################
#                                                                                                     #
#                                       Budget categories                                             #
#                                                                                                     #
#######################################################################################################

def BG_create_budget_category_rec(budget_ID = str, budget_cat_ID = str, name = str, string_constant = str, num_years = int):
    
    budget_cat_rec = {
        bgmvf.BG_BUDGET_CAT_BUDGET_ID                  : budget_ID,
        bgmvf.BG_BUDGET_CAT_ID                         : budget_cat_ID,
        bgmvf.BG_BUDGET_CAT_NAME                       : name,         
        bgmvf.BG_BUDGET_CAT_STR_CONST                  : string_constant,              
        bgmvf.BG_BUDGET_CAT_TOTAL_COST                 : np.full((num_years), 0.0).tolist(),
        bgmvf.BG_BUDGET_CAT_CUSTOM                     : False,
    }

    return budget_cat_rec

#######################################################################################################
#                                                                                                     #
#                                       Budget category shares                                        #
#                                                                                                     #
#######################################################################################################

def BG_create_budget_category_share_rec_PC(budget_ID = str, ind_type_ID = int, budget_cat_ID = str):
    budget_cat_share_rec = BG_create_budget_category_share_rec(budget_ID, gbc.GB_PC, ind_type_ID, str(gbc.GB_NOT_FOUND), budget_cat_ID)

    # For clarity, use theses three and never the indID field for PC indicators
    budget_cat_share_rec[bgmvf.BG_BUDGET_CAT_SHARE_COSTING_SECT_ID_PC] = str(gbc.GB_NOT_FOUND)
    budget_cat_share_rec[bgmvf.BG_BUDGET_CAT_SHARE_COSTING_SUBSECT_ID_PC] = str(gbc.GB_NOT_FOUND)
    budget_cat_share_rec[bgmvf.BG_BUDGET_CAT_SHARE_ACT_ID_PC] = str(gbc.GB_NOT_FOUND)

    # If IHT, then the PC indicators will vary by programme area as well.
    budget_cat_share_rec[bgmvf.BG_BUDGET_CAT_SHARE_PROG_AREA_ID_PC] = str(gbc.GB_NOT_FOUND)

    return budget_cat_share_rec

def BG_create_budget_category_share_rec_HS(budget_ID = str, ind_type_ID = int, budget_cat_ID = str):
    budget_cat_share_rec = BG_create_budget_category_share_rec(budget_ID, gbc.GB_HS, ind_type_ID, str(gbc.GB_NOT_FOUND), budget_cat_ID)

    budget_cat_share_rec[bgmvf.BG_BUDGET_CAT_SHARE_COSTING_SECT_HS_COSTING_ID] = str(gbc.GB_NOT_FOUND)
    budget_cat_share_rec[bgmvf.BG_BUDGET_CAT_SHARE_COSTING_SUBSECT_HS_COSTING_ID] = str(gbc.GB_NOT_FOUND)
    budget_cat_share_rec[bgmvf.BG_BUDGET_CAT_SHARE_ACT_HS_COSTING_ID] = str(gbc.GB_NOT_FOUND)

    budget_cat_share_rec[bgmvf.BG_BUDGET_CAT_SHARE_COSTING_SECT_HS_PROG_MGMT_ID] = str(gbc.GB_NOT_FOUND)
    budget_cat_share_rec[bgmvf.BG_BUDGET_CAT_SHARE_COSTING_SUBSECT_HS_PROG_MGMT_ID] = str(gbc.GB_NOT_FOUND)
    budget_cat_share_rec[bgmvf.BG_BUDGET_CAT_SHARE_ACT_HS_PROG_MGMT_ID] = str(gbc.GB_NOT_FOUND)

    return budget_cat_share_rec

def BG_create_budget_category_share_rec_GV(budget_ID = str, ind_type_ID = int, budget_cat_ID = str):
    budget_cat_share_rec = BG_create_budget_category_share_rec(budget_ID, gbc.GB_GV, ind_type_ID, str(gbc.GB_NOT_FOUND), budget_cat_ID)

    budget_cat_share_rec[bgmvf.BG_BUDGET_CAT_SHARE_COSTING_SECT_GV_COSTING_ID] = str(gbc.GB_NOT_FOUND)
    budget_cat_share_rec[bgmvf.BG_BUDGET_CAT_SHARE_COSTING_SUBSECT_GV_COSTING_ID] = str(gbc.GB_NOT_FOUND)
    budget_cat_share_rec[bgmvf.BG_BUDGET_CAT_SHARE_ACT_GV_COSTING_ID] = str(gbc.GB_NOT_FOUND)

    budget_cat_share_rec[bgmvf.BG_BUDGET_CAT_SHARE_COSTING_SECT_GV_PROG_MGMT_ID] = str(gbc.GB_NOT_FOUND)
    budget_cat_share_rec[bgmvf.BG_BUDGET_CAT_SHARE_COSTING_SUBSECT_GV_PROG_MGMT_ID] = str(gbc.GB_NOT_FOUND)
    budget_cat_share_rec[bgmvf.BG_BUDGET_CAT_SHARE_ACT_GV_PROG_MGMT_ID] = str(gbc.GB_NOT_FOUND)

    return budget_cat_share_rec

def BG_create_budget_category_share_rec_HF(budget_ID = str, ind_type_ID = int, budget_cat_ID = str):
    budget_cat_share_rec = BG_create_budget_category_share_rec(budget_ID, gbc.GB_HF, ind_type_ID, str(gbc.GB_NOT_FOUND), budget_cat_ID)

    budget_cat_share_rec[bgmvf.BG_BUDGET_CAT_SHARE_COSTING_SECT_HF_COSTING_ID] = str(gbc.GB_NOT_FOUND)
    budget_cat_share_rec[bgmvf.BG_BUDGET_CAT_SHARE_COSTING_SUBSECT_HF_COSTING_ID] = str(gbc.GB_NOT_FOUND)
    budget_cat_share_rec[bgmvf.BG_BUDGET_CAT_SHARE_ACT_HF_COSTING_ID] = str(gbc.GB_NOT_FOUND)

    budget_cat_share_rec[bgmvf.BG_BUDGET_CAT_SHARE_COSTING_SECT_HF_PROG_MGMT_ID] = str(gbc.GB_NOT_FOUND)
    budget_cat_share_rec[bgmvf.BG_BUDGET_CAT_SHARE_COSTING_SUBSECT_HF_PROG_MGMT_ID] = str(gbc.GB_NOT_FOUND)
    budget_cat_share_rec[bgmvf.BG_BUDGET_CAT_SHARE_ACT_HF_PROG_MGMT_ID] = str(gbc.GB_NOT_FOUND)

    return budget_cat_share_rec

def BG_create_budget_category_share_rec_IC(budget_ID = str, ind_type_ID = int, ind_ID = str, budget_cat_ID = str):
    budget_cat_share_rec = BG_create_budget_category_share_rec(budget_ID, gbc.GB_IC, ind_type_ID, ind_ID, budget_cat_ID)

    budget_cat_share_rec[bgmvf.BG_BUDGET_CAT_SHARE_DELIV_CHAN_ID] = str(gbc.GB_NOT_FOUND)

    # In IHT specific-plan mode, logistics and supply chain costs and wastage costs can be mapped by prog area. Otherwise, this field is not applicable.
    budget_cat_share_rec[bgmvf.BG_BUDGET_CAT_SHARE_PROG_AREA_ID_PC] = str(gbc.GB_NOT_FOUND)

    return budget_cat_share_rec

def BG_create_budget_category_share_rec_HW(budget_ID = str, ind_type_ID = int, ind_ID = str, budget_cat_ID = str):
    budget_cat_share_rec = BG_create_budget_category_share_rec(budget_ID, gbc.GB_HW, ind_type_ID, ind_ID, budget_cat_ID)

    budget_cat_share_rec[bgmvf.BG_BUDGET_CAT_SHARE_STAFF_LEVEL_ID_HW] = str(gbc.GB_NOT_FOUND) # equivalent to all staffing levels
    budget_cat_share_rec[bgmvf.BG_BUDGET_CAT_SHARE_COMPENS_TYPE_ID_HW] = str(gbc.GB_NOT_FOUND) # equivalent to all compensation types

    # For programme management indicators, use these instead of ind_ID, staff_level_ID, and compens_type_ID. Pass in str(gbc.GB_NOT_FOUND) 
    # for ind_ID on creation. 
    budget_cat_share_rec[bgmvf.BG_BUDGET_CAT_SHARE_COSTING_SECT_ID_PC] = str(gbc.GB_NOT_FOUND)
    budget_cat_share_rec[bgmvf.BG_BUDGET_CAT_SHARE_COSTING_SUBSECT_ID_PC] = str(gbc.GB_NOT_FOUND)
    budget_cat_share_rec[bgmvf.BG_BUDGET_CAT_SHARE_ACT_ID_PC] = str(gbc.GB_NOT_FOUND)

    return budget_cat_share_rec

def BG_create_budget_category_share_rec_IS(budget_ID = str, ind_type_ID = int, ind_ID = str, budget_cat_ID = str):
    budget_cat_share_rec = BG_create_budget_category_share_rec(budget_ID, gbc.GB_IS, ind_type_ID, ind_ID, budget_cat_ID)

    # For programme management indicators, use these instead of ind_ID. Most IS indicators vary by facility type, so we have an ind_ID param.
    # If it's a programme management indicator, just pass str(gbc.GB_NOT_FOUND) for ind_ID on creation and set these fields appropriately after.
    budget_cat_share_rec[bgmvf.BG_BUDGET_CAT_SHARE_COSTING_SECT_ID_PC] = str(gbc.GB_NOT_FOUND)
    budget_cat_share_rec[bgmvf.BG_BUDGET_CAT_SHARE_COSTING_SUBSECT_ID_PC] = str(gbc.GB_NOT_FOUND)
    budget_cat_share_rec[bgmvf.BG_BUDGET_CAT_SHARE_ACT_ID_PC] = str(gbc.GB_NOT_FOUND)

    # In IHT specific-plan mode, medical equipment and medical equipment maintenance and calibration costs can be mapped by prog area. Otherwise, this field is not applicable.
    budget_cat_share_rec[bgmvf.BG_BUDGET_CAT_SHARE_PROG_AREA_ID_PC] = str(gbc.GB_NOT_FOUND)

    return budget_cat_share_rec

def BG_create_budget_category_share_rec(budget_ID = str, mod_ID = int, ind_type_ID = int, ind_ID = str, budget_cat_ID = str):
    
    budget_cat_share_rec = {
        bgmvf.BG_BUDGET_CAT_SHARE_BUDGET_ID     : budget_ID,
        bgmvf.BG_BUDGET_CAT_SHARE_MOD_ID        : mod_ID,
        bgmvf.BG_BUDGET_CAT_SHARE_IND_TYPE_ID   : ind_type_ID, 
        bgmvf.BG_BUDGET_CAT_SHARE_IND_ID        : ind_ID,
        bgmvf.BG_BUDGET_CAT_SHARE_BUDGET_CAT_ID : budget_cat_ID,
        bgmvf.BG_BUDGET_CAT_SHARE_VALUE         : 0.0,
    }

    return budget_cat_share_rec

#######################################################################################################
#                                                                                                     #
#                                       Funding frameworks                                            #
#                                                                                                     #
#######################################################################################################

def BG_create_funding_framework_rec(ID = str, name = str, string_constant = str):

    funding_framework_rec = {
        bgmvf.BG_FUND_FW_ID           : ID,                          
        bgmvf.BG_FUND_FW_NAME         : name,   
        bgmvf.BG_FUND_FW_STR_CONSTANT : string_constant,  
        bgmvf.BG_FUND_FW_CUSTOM       : False,                
    }

    return funding_framework_rec

#######################################################################################################
#                                                                                                     #
#                                       Funding sources                                               #
#                                                                                                     #
#######################################################################################################

def BG_create_funding_source_rec(fund_framework_ID = str, fund_source_ID = str, name = str, string_constant = str):
    
    funding_source_rec = {
        bgmvf.BG_FUND_SOURCE_FW_ID        : fund_framework_ID,
        bgmvf.BG_FUND_SOURCE_ID           : fund_source_ID,
        bgmvf.BG_FUND_SOURCE_NAME         : name,     
        bgmvf.BG_FUND_SOURCE_STR_CONSTANT : string_constant,     
        bgmvf.BG_FUND_SOURCE_CUSTOM       : False,             
    }

    return funding_source_rec

#######################################################################################################
#                                                                                                     #
#                                       Funding source shares                                         #
#                                                                                                     #
#######################################################################################################

def BG_create_funding_source_share_rec_PC(budget_ID = str, ind_type_ID = int, fund_framework_ID = str, fund_source_ID = str):
    
    funding_source_share_rec = BG_create_funding_source_share_rec(budget_ID, gbc.GB_PC, ind_type_ID, str(gbc.GB_NOT_FOUND), fund_framework_ID, fund_source_ID)

    # For clarity, use theses three and never the indID field for PC indicators
    funding_source_share_rec[bgmvf.BG_FUND_SOURCE_SHARE_COSTING_SECT_ID_PC] = str(gbc.GB_NOT_FOUND)
    funding_source_share_rec[bgmvf.BG_FUND_SOURCE_SHARE_COSTING_SUBSECT_ID_PC] = str(gbc.GB_NOT_FOUND)
    funding_source_share_rec[bgmvf.BG_FUND_SOURCE_SHARE_ACT_ID_PC] = str(gbc.GB_NOT_FOUND)

    # If IHT, then the PC indicators will vary by programme area as well.
    funding_source_share_rec[bgmvf.BG_FUND_SOURCE_SHARE_PROG_AREA_ID_PC] = str(gbc.GB_NOT_FOUND)

    return funding_source_share_rec

def BG_create_funding_source_share_rec_HS(budget_ID = str, ind_type_ID = int, fund_framework_ID = str, fund_source_ID = str):
    
    funding_source_share_rec = BG_create_funding_source_share_rec(budget_ID, gbc.GB_HS, ind_type_ID, str(gbc.GB_NOT_FOUND), fund_framework_ID, fund_source_ID)

    funding_source_share_rec[bgmvf.BG_FUND_SOURCE_SHARE_COSTING_SECT_HS_COSTING_ID] = str(gbc.GB_NOT_FOUND)
    funding_source_share_rec[bgmvf.BG_FUND_SOURCE_SHARE_COSTING_SUBSECT_HS_COSTING_ID] = str(gbc.GB_NOT_FOUND)
    funding_source_share_rec[bgmvf.BG_FUND_SOURCE_SHARE_ACT_HS_COSTING_ID] = str(gbc.GB_NOT_FOUND)

    funding_source_share_rec[bgmvf.BG_FUND_SOURCE_SHARE_COSTING_SECT_HS_PROG_MGMT_ID] = str(gbc.GB_NOT_FOUND)
    funding_source_share_rec[bgmvf.BG_FUND_SOURCE_SHARE_COSTING_SUBSECT_HS_PROG_MGMT_ID] = str(gbc.GB_NOT_FOUND)
    funding_source_share_rec[bgmvf.BG_FUND_SOURCE_SHARE_ACT_HS_PROG_MGMT_ID] = str(gbc.GB_NOT_FOUND)

    return funding_source_share_rec

def BG_create_funding_source_share_rec_GV(budget_ID = str, ind_type_ID = int, fund_framework_ID = str, fund_source_ID = str):
    
    funding_source_share_rec = BG_create_funding_source_share_rec(budget_ID, gbc.GB_GV, ind_type_ID, str(gbc.GB_NOT_FOUND), fund_framework_ID, fund_source_ID)

    funding_source_share_rec[bgmvf.BG_FUND_SOURCE_SHARE_COSTING_SECT_GV_COSTING_ID] = str(gbc.GB_NOT_FOUND)
    funding_source_share_rec[bgmvf.BG_FUND_SOURCE_SHARE_COSTING_SUBSECT_GV_COSTING_ID] = str(gbc.GB_NOT_FOUND)
    funding_source_share_rec[bgmvf.BG_FUND_SOURCE_SHARE_ACT_GV_COSTING_ID] = str(gbc.GB_NOT_FOUND)

    funding_source_share_rec[bgmvf.BG_FUND_SOURCE_SHARE_COSTING_SECT_GV_PROG_MGMT_ID] = str(gbc.GB_NOT_FOUND)
    funding_source_share_rec[bgmvf.BG_FUND_SOURCE_SHARE_COSTING_SUBSECT_GV_PROG_MGMT_ID] = str(gbc.GB_NOT_FOUND)
    funding_source_share_rec[bgmvf.BG_FUND_SOURCE_SHARE_ACT_GV_PROG_MGMT_ID] = str(gbc.GB_NOT_FOUND)

    return funding_source_share_rec

def BG_create_funding_source_share_rec_HF(budget_ID = str, ind_type_ID = int, fund_framework_ID = str, fund_source_ID = str):
    
    funding_source_share_rec = BG_create_funding_source_share_rec(budget_ID, gbc.GB_HF, ind_type_ID, str(gbc.GB_NOT_FOUND), fund_framework_ID, fund_source_ID)

    funding_source_share_rec[bgmvf.BG_FUND_SOURCE_SHARE_COSTING_SECT_HF_COSTING_ID] = str(gbc.GB_NOT_FOUND)
    funding_source_share_rec[bgmvf.BG_FUND_SOURCE_SHARE_COSTING_SUBSECT_HF_COSTING_ID] = str(gbc.GB_NOT_FOUND)
    funding_source_share_rec[bgmvf.BG_FUND_SOURCE_SHARE_ACT_HF_COSTING_ID] = str(gbc.GB_NOT_FOUND)

    funding_source_share_rec[bgmvf.BG_FUND_SOURCE_SHARE_COSTING_SECT_HF_PROG_MGMT_ID] = str(gbc.GB_NOT_FOUND)
    funding_source_share_rec[bgmvf.BG_FUND_SOURCE_SHARE_COSTING_SUBSECT_HF_PROG_MGMT_ID] = str(gbc.GB_NOT_FOUND)
    funding_source_share_rec[bgmvf.BG_FUND_SOURCE_SHARE_ACT_HF_PROG_MGMT_ID] = str(gbc.GB_NOT_FOUND)

    return funding_source_share_rec

def BG_create_funding_source_share_rec_IC(budget_ID = str, ind_type_ID = int, ind_ID = int, fund_framework_ID = str, fund_source_ID = str):
    
    funding_source_share_rec = BG_create_funding_source_share_rec(budget_ID, gbc.GB_IC, ind_type_ID, ind_ID, fund_framework_ID, fund_source_ID)

    funding_source_share_rec[bgmvf.BG_FUND_SOURCE_SHARE_DELIV_CHAN_ID] = str(gbc.GB_NOT_FOUND)

    # In IHT specific-plan mode, logistics and supply chain costs and wastage costs can be mapped by prog area. Otherwise, this field is not applicable.
    funding_source_share_rec[bgmvf.BG_FUND_SOURCE_SHARE_PROG_AREA_ID_PC] = str(gbc.GB_NOT_FOUND)

    return funding_source_share_rec

def BG_create_funding_source_share_rec_HW(budget_ID = str, ind_type_ID = int, ind_ID = int, fund_framework_ID = str, fund_source_ID = str):
    
    funding_source_share_rec = BG_create_funding_source_share_rec(budget_ID, gbc.GB_HW, ind_type_ID, ind_ID, fund_framework_ID, fund_source_ID)

    funding_source_share_rec[bgmvf.BG_FUND_SOURCE_SHARE_STAFF_LEVEL_ID_HW] = str(gbc.GB_NOT_FOUND) # equivalent to all staffing levels
    funding_source_share_rec[bgmvf.BG_FUND_SOURCE_SHARE_COMPENS_TYPE_ID_HW] = str(gbc.GB_NOT_FOUND) # equivalent to all compensation types

    # For programme management indicators, use these instead of ind_ID, staff_level_ID, and compens_type_ID. Pass in str(gbc.GB_NOT_FOUND) 
    # for ind_ID on creation. 
    funding_source_share_rec[bgmvf.BG_FUND_SOURCE_SHARE_COSTING_SECT_ID_PC] = str(gbc.GB_NOT_FOUND)
    funding_source_share_rec[bgmvf.BG_FUND_SOURCE_SHARE_COSTING_SUBSECT_ID_PC] = str(gbc.GB_NOT_FOUND)
    funding_source_share_rec[bgmvf.BG_FUND_SOURCE_SHARE_ACT_ID_PC] = str(gbc.GB_NOT_FOUND)

    return funding_source_share_rec

def BG_create_funding_source_share_rec_IS(budget_ID = str, ind_type_ID = int, ind_ID = int, fund_framework_ID = str, fund_source_ID = str):
    
    funding_source_share_rec = BG_create_funding_source_share_rec(budget_ID, gbc.GB_IS, ind_type_ID, ind_ID, fund_framework_ID, fund_source_ID)

    # For programme management indicators, use these instead of ind_ID. Most IS indicators vary by facility type, so we have an ind_ID param.
    # If it's a programme management indicator, just pass str(gbc.GB_NOT_FOUND) for ind_ID on creation and set these fields appropriately after.
    funding_source_share_rec[bgmvf.BG_FUND_SOURCE_SHARE_COSTING_SECT_ID_PC] = str(gbc.GB_NOT_FOUND)
    funding_source_share_rec[bgmvf.BG_FUND_SOURCE_SHARE_COSTING_SUBSECT_ID_PC] = str(gbc.GB_NOT_FOUND)
    funding_source_share_rec[bgmvf.BG_FUND_SOURCE_SHARE_ACT_ID_PC] = str(gbc.GB_NOT_FOUND)

    # In IHT specific-plan mode, medical equipment and medical equipment maintenance and calibration costs can be mapped by prog area. Otherwise, this field is not applicable.
    funding_source_share_rec[bgmvf.BG_FUND_SOURCE_SHARE_PROG_AREA_ID_PC] = str(gbc.GB_NOT_FOUND)

    return funding_source_share_rec

def BG_create_funding_source_share_rec(budget_ID = str, mod_ID = int, ind_type_ID = int, ind_ID = int, fund_framework_ID = str, fund_source_ID = str):
    
    funding_source_share_rec = {
        bgmvf.BG_FUND_SOURCE_SHARE_BUDGET_ID      : budget_ID,
        bgmvf.BG_FUND_SOURCE_SHARE_MOD_ID         : mod_ID,
        bgmvf.BG_FUND_SOURCE_SHARE_IND_TYPE_ID    : ind_type_ID, 
        bgmvf.BG_FUND_SOURCE_SHARE_IND_ID         : ind_ID,
        bgmvf.BG_FUND_SOURCE_SHARE_FW_ID          : fund_framework_ID,
        bgmvf.BG_FUND_SOURCE_SHARE_FUND_SOURCE_ID : fund_source_ID,
        bgmvf.BG_FUND_SOURCE_SHARE_VALUE          : 0.0,
    }

    return funding_source_share_rec

#######################################################################################################
#                                                                                                     #
#                                       Indicators                                                    #
#                                                                                                     #
#######################################################################################################

def BG_create_indicator_rec_PC(budget_ID = str, ind_type_ID = int, budget_cat_ID = str, name = str):
    ind_rec = BG_create_indicator_rec(budget_ID, gbc.GB_PC, ind_type_ID, str(gbc.GB_NOT_FOUND), budget_cat_ID, name)

    # For clarity, use theses three and never the indID field for PC indicators
    ind_rec[bgmvf.BG_IND_COSTING_SECT_ID_PC] = str(gbc.GB_NOT_FOUND)
    ind_rec[bgmvf.BG_IND_COSTING_SUBSECT_ID_PC] = str(gbc.GB_NOT_FOUND)
    ind_rec[bgmvf.BG_IND_ACT_ID_PC] = str(gbc.GB_NOT_FOUND)

    # If IHT, then the PC indicators will vary by programme area as well.
    ind_rec[bgmvf.BG_IH_F_PROG_AREA_ID] = str(gbc.GB_NOT_FOUND)

    return ind_rec

def BG_create_indicator_rec_HS(budget_ID = str, ind_type_ID = int, budget_cat_ID = str, name = str):
    ind_rec = BG_create_indicator_rec(budget_ID, gbc.GB_HS, ind_type_ID, str(gbc.GB_NOT_FOUND), budget_cat_ID, name)

    ind_rec[bgmvf.BG_IND_COSTING_SECT_HS_COSTING_ID] = str(gbc.GB_NOT_FOUND)
    ind_rec[bgmvf.BG_IND_COSTING_SUBSECT_HS_COSTING_ID] = str(gbc.GB_NOT_FOUND)
    ind_rec[bgmvf.BG_IND_ACT_HS_COSTING_ID] = str(gbc.GB_NOT_FOUND)

    ind_rec[bgmvf.BG_IND_COSTING_SECT_HS_PROG_MGMT_ID] = str(gbc.GB_NOT_FOUND)
    ind_rec[bgmvf.BG_IND_COSTING_SUBSECT_HS_PROG_MGMT_ID] = str(gbc.GB_NOT_FOUND)
    ind_rec[bgmvf.BG_IND_ACT_HS_PROG_MGMT_ID] = str(gbc.GB_NOT_FOUND)

    return ind_rec

def BG_create_indicator_rec_GV(budget_ID = str, ind_type_ID = int, budget_cat_ID = str, name = str):
    ind_rec = BG_create_indicator_rec(budget_ID, gbc.GB_GV, ind_type_ID, str(gbc.GB_NOT_FOUND), budget_cat_ID, name)

    ind_rec[bgmvf.BG_IND_COSTING_SECT_GV_COSTING_ID] = str(gbc.GB_NOT_FOUND)
    ind_rec[bgmvf.BG_IND_COSTING_SUBSECT_GV_COSTING_ID] = str(gbc.GB_NOT_FOUND)
    ind_rec[bgmvf.BG_IND_ACT_GV_COSTING_ID] = str(gbc.GB_NOT_FOUND)

    ind_rec[bgmvf.BG_IND_COSTING_SECT_GV_PROG_MGMT_ID] = str(gbc.GB_NOT_FOUND)
    ind_rec[bgmvf.BG_IND_COSTING_SUBSECT_GV_PROG_MGMT_ID] = str(gbc.GB_NOT_FOUND)
    ind_rec[bgmvf.BG_IND_ACT_GV_PROG_MGMT_ID] = str(gbc.GB_NOT_FOUND)

    return ind_rec

def BG_create_indicator_rec_HF(budget_ID = str, ind_type_ID = int, budget_cat_ID = str, name = str):
    ind_rec = BG_create_indicator_rec(budget_ID, gbc.GB_HF, ind_type_ID, str(gbc.GB_NOT_FOUND), budget_cat_ID, name)

    ind_rec[bgmvf.BG_IND_COSTING_SECT_HF_COSTING_ID] = str(gbc.GB_NOT_FOUND)
    ind_rec[bgmvf.BG_IND_COSTING_SUBSECT_HF_COSTING_ID] = str(gbc.GB_NOT_FOUND)
    ind_rec[bgmvf.BG_IND_ACT_HF_COSTING_ID] = str(gbc.GB_NOT_FOUND)

    ind_rec[bgmvf.BG_IND_COSTING_SECT_HF_PROG_MGMT_ID] = str(gbc.GB_NOT_FOUND)
    ind_rec[bgmvf.BG_IND_COSTING_SUBSECT_HF_PROG_MGMT_ID] = str(gbc.GB_NOT_FOUND)
    ind_rec[bgmvf.BG_IND_ACT_HF_PROG_MGMT_ID] = str(gbc.GB_NOT_FOUND)

    return ind_rec

def BG_create_indicator_rec_IC(budget_ID = str, ind_type_ID = int, ind_ID = int, budget_cat_ID = str, name = str):
    # ind_ID should be an intervention or drug/supply ID.
    ind_rec = BG_create_indicator_rec(budget_ID, gbc.GB_IC, ind_type_ID, ind_ID, budget_cat_ID, name)

    ind_rec[bgmvf.BG_IND_DELIV_CHAN_ID_IC] = str(gbc.GB_NOT_FOUND) # equivalent to all delivery channels (OR none for indicators where delivery channel is irrelevant)

    # In IHT specific-plan mode, logistics and supply chain costs and wastage costs can be mapped by prog area. Otherwise, this field is not applicable.
    ind_rec[bgmvf.BG_IND_PROG_AREA_ID_PC] = str(gbc.GB_NOT_FOUND)

    return ind_rec

def BG_create_indicator_rec_HW(budget_ID = str, ind_type_ID = int, ind_ID = int, budget_cat_ID = str, name = str):
    # ind_ID should be a staff type ID.
    ind_rec = BG_create_indicator_rec(budget_ID, gbc.GB_HW, ind_type_ID, ind_ID, budget_cat_ID, name)

    ind_rec[bgmvf.BG_IND_STAFF_LEVEL_ID_HW] = gbc.GB_NOT_FOUND # equivalent to all staffing levels
    ind_rec[bgmvf.BG_IND_COMPENS_TYPE_ID_HW] = gbc.GB_NOT_FOUND # equivalent to all compensation types

    # For programme management indicators, use these instead of ind_ID, staff_level_ID, and compens_type_ID. Pass in str(gbc.GB_NOT_FOUND) 
    # for ind_ID on creation. 
    ind_rec[bgmvf.BG_IND_COSTING_SECT_ID_PC] = str(gbc.GB_NOT_FOUND)
    ind_rec[bgmvf.BG_IND_COSTING_SUBSECT_ID_PC] = str(gbc.GB_NOT_FOUND)
    ind_rec[bgmvf.BG_IND_ACT_ID_PC] = str(gbc.GB_NOT_FOUND)

    return ind_rec

def BG_create_indicator_rec_IS(budget_ID = str, ind_type_ID = int, ind_ID = int, budget_cat_ID = str, name = str):
    # ind_ID should be a facility type ID.
    ind_rec = BG_create_indicator_rec(budget_ID, gbc.GB_IS, ind_type_ID, ind_ID, budget_cat_ID, name)

    # For programme management indicators, use these instead of ind_ID. Most IS indicators vary by facility type, so we have an ind_ID param.
    # If it's a programme management indicator, just pass str(gbc.GB_NOT_FOUND) for ind_ID on creation and set these fields appropriately after.
    ind_rec[bgmvf.BG_IND_COSTING_SECT_ID_PC] = str(gbc.GB_NOT_FOUND)
    ind_rec[bgmvf.BG_IND_COSTING_SUBSECT_ID_PC] = str(gbc.GB_NOT_FOUND)
    ind_rec[bgmvf.BG_IND_ACT_ID_PC] = str(gbc.GB_NOT_FOUND)

    # In IHT specific-plan mode, medical equipment and medical equipment maintenance and calibration costs can be mapped by prog area. Otherwise, this field is not applicable.
    ind_rec[bgmvf.BG_FUND_SOURCE_SHARE_PROG_AREA_ID_PC] = str(gbc.GB_NOT_FOUND)

    return ind_rec

def BG_create_indicator_rec(budget_ID = str, mod_ID = int, ind_type_ID = int, ind_ID = int, budget_cat_ID = str, name = str):
    
    ind_rec = {
        bgmvf.BG_IND_BUDGET_ID                  : budget_ID,
        # bgmvf.BG_IND_FUND_FW_ID                 : fund_framework_ID, 
        bgmvf.BG_IND_MOD_ID                     : mod_ID,
        bgmvf.BG_IND_TYPE_ID                    : ind_type_ID,
        bgmvf.BG_IND_ID                         : ind_ID,
        bgmvf.BG_IND_BUDGET_CAT_ID              : budget_cat_ID,
        # bgmvf.BG_IND_FUND_SOURCE_ID             : fund_source_ID, 
        bgmvf.BG_IND_NAME                       : name,
        #bgmvf.BG_IND_IS_AGGREGATE               : False, # stopped using this in favor of using str(gbc.GB_NOT_FOUND) as the indicator ID for all items combined.                                                                                                                                                                                               
        #bgmvf.BG_IND_DISTRIB_MULTI_CATS         : False,
        #bgmvf.BG_IND_DISTRIB_MULTI_FUND_SOURCES : False,
        bgmvf.BG_IND_FUND_SOURCE_BY_FRAMEWORK   : {},
    }

    return ind_rec

#######################################################################################################
#                                                                                                     #
#                                            Quick mapping                                            #
#                                                                                                     #
#######################################################################################################

def BG_create_quick_map_rec(budget_ID = str, mod_ID = int, target_ID = int, ind_type_ID = int):
    
    quick_map_rec = {
        bgmvf.BG_QUICK_MAP_BUDGET_ID      : budget_ID,
        bgmvf.BG_QUICK_MAP_MOD_ID         : mod_ID,
        bgmvf.BG_QUICK_MAP_TARGET_ID      : target_ID,
        bgmvf.BG_QUICK_MAP_IND_TYPE_ID    : ind_type_ID,
        bgmvf.BG_QUICK_MAP_IS_AGGREGATE   : False,        
        bgmvf.BG_QUICK_MAP_ON             : False,                                                                                                                                                                                                
    }

    # if mod_ID == gbc.GB_IC:
    #     quick_map_rec[bgmvf.BG_QUICK_MAP_DELIV_CHAN_ID_IC] = gbc.GB_NOT_FOUND

    return quick_map_rec

#######################################################################################################
#                                                                                                     #
#                                            Results                                                  #
#                                                                                                     #
#######################################################################################################

def BG_init_budget_cat_year_arr(num_budgets = int, num_budget_cats = int, num_years = int, value = 0.0):
     return np.full((num_budgets, num_budget_cats, num_years), value, gbc.GB_CALC_FLOAT_TYPE)

def BG_init_budget_fund_source_year_arr(num_budgets = int, num_fund_frameworks = int, num_funding_sources = int, num_years = int, value = 0.0):
     return np.full((num_budgets, num_fund_frameworks, num_funding_sources, num_years), value, gbc.GB_CALC_FLOAT_TYPE)

def BG_init_budget_cat_fund_source_res_rec(budget_ID = str, fund_framework_ID = str, fund_source_ID = str, budget_cat_ID = str, num_years = int):
    return {
        bgmvf.BG_BUDGET_ID      : budget_ID,
        bgmvf.BG_FUND_FW_ID     : fund_framework_ID,
        bgmvf.BG_FUND_SOURCE_ID : fund_source_ID,
        bgmvf.BG_BUDGET_CAT_ID  : budget_cat_ID,
        bgmvf.BG_F_VALUE        : np.full((num_years), 0.0, gbc.GB_CALC_FLOAT_TYPE)
    }


# Input ModVars
LG_VERSION_TAG = "<LG_Version_V1>"
LG_DATA_CHANGED_TAG = "<LG_DataChanged_V1>"
LG_FILE_NAME_TAG = "<LG_FileName_V1>"
LG_PROJ_VALID_TAG = "<LG_ProjectionValid_V1>"
LG_PROJ_VALID_DATE_TAG = "<LG_ProjectionValidDate_V1>"
LG_SUPPLY_CHAIN_LEVEL_RECORDS_TAG = "<LG_SupplyChainLevelRecords_V1>"
LG_WAREHOUSE_RECORDS_TAG = "<LG_WarehouseRecords_V1>"
LG_DISAGGREGATION_LEVEL_TAG = "<LG_DisaggregationLevel_V1>"
LG_CALC_APPROACH_TAG = "<LG_CalcApproach_V1>"
LG_VEHICLE_TYPE_RECORDS_TAG = "<LG_VehicleTypeRecords_V1>"
LG_WORKER_TYPE_RECORDS_TAG = "<LG_WorkerTypeRecords_V1>"
LG_COLD_CHAIN_RECORDS_TAG = "<LG_ColdChainRecords_V1>"
LG_DEFAULT_COLD_CHAIN_RECORDS_MEGA_TAG = "<LG_DefaultColdChainRecordsMeta_V1>"
LG_COLD_CHAIN_TARG_CONFIG_OPTION_TAG = "<LG_ColdChainTargetConfigOption_V1>"
LG_THIRD_PARTY_LOG_CONTRACT_RECORDS_TAG = "<LG_ThirdPartyLogisticsContractRecords_V1>"
LG_ENTER_COMP_MAINT_OP_COSTS_VEHICLES_TAG = "<LG_EnterComponentMaintOpCostsVehicles_V1>"
LG_COST_FUEL_PER_LITER_TAG = "<LG_CostOfFuelPerLiter_V1>"
LG_ENTER_COMP_CONSTRUCT_COSTS_WAREHOUSES_TAG = "<LG_EnterComponentConstructCostsWarehouses_V1>"
LG_ENTER_COMP_OP_COSTS_WAREHOUSES_TAG = "<LG_EnterComponentOpCostsWarehouses_V1>"
LG_VEHICLE_TARG_CONFIG_OPTION_TAG = "<LG_VehicleTargetConfigOption_V1>"
LG_VEHICLE_TARG_VARY_TRIPS_PER_YEAR_TAG = "<LG_VehicleTargetVaryTripsPerYear_V1>"
LG_VEHICLE_PERFORMANCE_CONFIG_OPTION_TAG = "<LG_VehiclePerformanceConfigOption_V1>"
LG_NUM_WORK_HOURS_PER_DAY = "<LG_NumWorkHoursPerDay_V1>"
LG_NUM_WORK_DAYS_PER_YEAR = "<LG_NumWorkDaysPerYear_V1>"

# Output / Result ModVars

# Warehouses
LG_RES_NUM_WAREHOUSES_BY_LEVEL = "<LG_ResNumWarehousesByLevel_V1>"
LG_RES_NUM_WAREHOUSES_BY_WAREHOUSE = "<LG_ResNumWarehousesByWarehouse_V1>"
LG_RES_NUM_WAREHOUSES_BY_WAREHOUSE_BY_LEVEL = "<LG_ResNumWarehousesByWarehouseByLevel_V1>"
LG_RES_OPERATING_COSTS_BY_LEVEL = "<LG_ResWarehouseOperatingCostsByLevel_V1>"
LG_RES_OPERATING_COSTS_BY_WAREHOUSE = "<LG_ResWarehouseOperatingCostsByWarehouse_V1>"
LG_RES_OPERATING_COSTS_BY_WAREHOUSE_BY_LEVEL = "<LG_ResWarehouseOperatingCostsByWarehouseByLevel_V1>"
LG_RES_CONSTRUCTION_COSTS_BY_LEVEL = "<LG_ResWarehouseConstructionCostsByLevel_V1>"
LG_RES_CONSTRUCTION_COSTS_BY_WAREHOUSE = "<LG_ResWarehouseConstructionCostsByWarehouse_V1>"
LG_RES_CONSTRUCTION_COSTS_BY_WAREHOUSE_BY_LEVEL = "<LG_ResWarehouseConstructionCostsByWarehouseByLevel_V1>"
LG_RES_REHAB_COSTS_BY_LEVEL = "<LG_ResWarehouseRehabCostsByLevel_V1>"
LG_RES_REHAB_COSTS_BY_WAREHOUSE = "<LG_ResWarehouseRehabCostsByWarehouse_V1>"
LG_RES_REHAB_COSTS_BY_WAREHOUSE_BY_LEVEL = "<LG_ResWarehouseRehabCostsByWarehouseByLevel_V1>"
# Volume analytics
LG_RES_LABOR_UTILIZATION_BY_LEVEL = "<LG_ResLaborUtilizationByLevel_V1>"
LG_RES_LABOR_UTILIZATION_BY_WAREHOUSE = "<LG_ResLaborUtilizationByWarehouse_V1>"
LG_RES_LABOR_UTILIZATION_BY_WAREHOUSE_BY_LEVEL = "<LG_ResLaborUtilizationByWarehouseByLevel_V1>"
LG_RES_AVG_DEMAND_AMBIENT_BY_LEVEL = "<LG_ResAvgDemandAmbientByLevel_V1>"
LG_RES_AVG_DEMAND_AMBIENT_BY_WAREHOUSE = "<LG_ResAvgDemandAmbientByWarehouse_V1>"
LG_RES_AVG_DEMAND_AMBIENT_BY_WAREHOUSE_BY_LEVEL = "<LG_ResAvgDemandAmbientByWarehouseByLevel_V1>"
LG_RES_AVG_DEMAND_PER_PERIOD_AMBIENT_BY_LEVEL = "<LG_ResAvgDemandPerPeriodAmbientByLevel_V1>"
LG_RES_AVG_DEMAND_PER_PERIOD_AMBIENT_BY_WAREHOUSE = "<LG_ResAvgDemandPerPeriodAmbientByWarehouse_V1>"
LG_RES_AVG_DEMAND_PER_PERIOD_AMBIENT_BY_WAREHOUSE_BY_LEVEL = "<LG_ResAvgDemandPerPeriodAmbientByWarehouseByLevel_V1>"
LG_RES_AVG_SAFETY_STOCK_AMBIENT_BY_LEVEL = "<LG_ResAvgSafetyStockAmbientByLevel_V1>"
LG_RES_AVG_SAFETY_STOCK_AMBIENT_BY_WAREHOUSE = "<LG_ResAvgSafetyStockAmbientByWarehouse_V1>"
LG_RES_AVG_SAFETY_STOCK_AMBIENT_BY_WAREHOUSE_BY_LEVEL = "<LG_ResAvgSafetyStockAmbientByWarehouseByLevel_V1>"
LG_RES_EXPECTED_MAX_INVENTORY_AMBIENT_BY_LEVEL = "<LG_ResExpectedMaxInventoryAmbientByLevel_V1>"
LG_RES_EXPECTED_MAX_INVENTORY_AMBIENT_BY_WAREHOUSE = "<LG_ResExpectedMaxInventoryAmbientByWarehouse_V1>"
LG_RES_EXPECTED_MAX_INVENTORY_AMBIENT_BY_WAREHOUSE_BY_LEVEL = "<LG_ResExpectedMaxInventoryAmbientByWarehouseByLevel_V1>"
LG_RES_EXPECTED_AVG_INVENTORY_AMBIENT_BY_LEVEL = "<LG_ResExpectedAvgInventoryAmbientByLevel_V1>"
LG_RES_EXPECTED_AVG_INVENTORY_AMBIENT_BY_WAREHOUSE = "<LG_ResExpectedAvgInventoryAmbientByWarehouse_V1>"
LG_RES_EXPECTED_AVG_INVENTORY_AMBIENT_BY_WAREHOUSE_BY_LEVEL = "<LG_ResExpectedAvgInventoryAmbientByWarehouseByLevel_V1>"
LG_RES_AVG_CAPACITY_UTILIZATION_AMBIENT_BY_LEVEL = "<LG_ResAvgCapacityUtilizationAmbientByLevel_V1>"
LG_RES_AVG_CAPACITY_UTILIZATION_AMBIENT_BY_WAREHOUSE = "<LG_ResAvgCapacityUtilizationAmbientByWarehouse_V1>"
LG_RES_AVG_CAPACITY_UTILIZATION_AMBIENT_BY_WAREHOUSE_BY_LEVEL = (
    "<LG_ResAvgCapacityUtilizationAmbientByWarehouseByLevel_V1>"
)
LG_RES_COLD_CHAIN_CAPACITY_BY_LEVEL = "<LG_ResColdChainCapacityByLevel_V1>"
LG_RES_COLD_CHAIN_CAPACITY_BY_WAREHOUSE = "<LG_ResColdChainCapacityByWarehouse_V1>"
LG_RES_COLD_CHAIN_CAPACITY_BY_WAREHOUSE_BY_LEVEL = "<LG_ResColdChainCapacityByWarehouseByLevel_V1>"
LG_RES_AVG_DEMAND_COLD_CHAIN_BY_LEVEL = "<LG_ResAvgDemandColdChainByLevel_V1>"
LG_RES_AVG_DEMAND_COLD_CHAIN_BY_WAREHOUSE = "<LG_ResAvgDemandColdChainByWarehouse_V1>"
LG_RES_AVG_DEMAND_COLD_CHAIN_BY_WAREHOUSE_BY_LEVEL = "<LG_ResAvgDemandColdChainByWarehouseByLevel_V1>"
LG_RES_AVG_DEMAND_PER_PERIOD_COLD_CHAIN_BY_LEVEL = "<LG_ResAvgDemandPerPeriodColdChainByLevel_V1>"
LG_RES_AVG_DEMAND_PER_PERIOD_COLD_CHAIN_BY_WAREHOUSE = "<LG_ResAvgDemandPerPeriodColdChainByWarehouse_V1>"
LG_RES_AVG_DEMAND_PER_PERIOD_COLD_CHAIN_BY_WAREHOUSE_BY_LEVEL = (
    "<LG_ResAvgDemandPerPeriodColdChainByWarehouseByLevel_V1>"
)
LG_RES_AVG_SAFETY_STOCK_COLD_CHAIN_BY_LEVEL = "<LG_ResAvgSafetyStockColdChainByLevel_V1>"
LG_RES_AVG_SAFETY_STOCK_COLD_CHAIN_BY_WAREHOUSE = "<LG_ResAvgSafetyStockColdChainByWarehouse_V1>"
LG_RES_AVG_SAFETY_STOCK_COLD_CHAIN_BY_WAREHOUSE_BY_LEVEL = "<LG_ResAvgSafetyStockColdChainByWarehouseByLevel_V1>"
LG_RES_EXPECTED_MAX_INVENTORY_COLD_CHAIN_BY_LEVEL = "<LG_ResExpectedMaxInventoryColdChainByLevel_V1>"
LG_RES_EXPECTED_MAX_INVENTORY_COLD_CHAIN_BY_WAREHOUSE = "<LG_ResExpectedMaxInventoryColdChainByWarehouse_V1>"
LG_RES_EXPECTED_MAX_INVENTORY_COLD_CHAIN_BY_WAREHOUSE_BY_LEVEL = (
    "<LG_ResExpectedMaxInventoryColdChainByWarehouseByLevel_V1>"
)
LG_RES_EXPECTED_AVG_INVENTORY_COLD_CHAIN_BY_LEVEL = "<LG_ResExpectedAvgInventoryColdChainByLevel_V1>"
LG_RES_EXPECTED_AVG_INVENTORY_COLD_CHAIN_BY_WAREHOUSE = "<LG_ResExpectedAvgInventoryColdChainByWarehouse_V1>"
LG_RES_EXPECTED_AVG_INVENTORY_COLD_CHAIN_BY_WAREHOUSE_BY_LEVEL = (
    "<LG_ResExpectedAvgInventoryColdChainByWarehouseByLevel_V1>"
)
LG_RES_AVG_CAPACITY_UTILIZATION_COLD_CHAIN_BY_LEVEL = "<LG_ResAvgCapacityUtilizationColdChainByLevel_V1>"
LG_RES_AVG_CAPACITY_UTILIZATION_COLD_CHAIN_BY_WAREHOUSE = "<LG_ResAvgCapacityUtilizationColdChainByWarehouse_V1>"
LG_RES_AVG_CAPACITY_UTILIZATION_COLD_CHAIN_BY_WAREHOUSE_BY_LEVEL = (
    "<LG_ResAvgCapacityUtilizationColdChainByWarehouseByLevel_V1>"
)


# Vehicles
LG_RES_NUM_VEHICLES_AVAIL_BY_LEVEL = "<LG_ResNumVehiclesAvailByLevel_V1>"
LG_RES_NUM_VEHICLES_AVAIL_BY_WAREHOUSE = "<LG_ResNumVehiclesAvailByWarehouse_V1>"
LG_RES_NUM_VEHICLES_AVAIL_BY_WAREHOUSE_BY_LEVEL = "<LG_ResNumVehiclesAvailByWarehouseByLevel_V1>"
LG_RES_NUM_VEHICLES_PURCHASED_BY_LEVEL = "<LG_ResNumVehiclesPurchasedByLevel_V1>"
LG_RES_NUM_VEHICLES_PURCHASED_BY_WAREHOUSE = "<LG_ResNumVehiclesPurchasedByWarehouse_V1>"
LG_RES_NUM_VEHICLES_PURCHASED_BY_WAREHOUSE_BY_LEVEL = "<LG_ResNumVehiclesPurchasedByWarehouseByLevel_V1>"
LG_RES_VEHICLE_PURCHASE_COSTS_BY_LEVEL = "<LG_ResVehiclePurchaseCostsByLevel_V1>"
LG_RES_VEHICLE_PURCHASE_COSTS_BY_WAREHOUSE = "<LG_ResVehiclePurchaseCostsByWarehouse_V1>"
LG_RES_VEHICLE_PURCHASE_COSTS_BY_WAREHOUSE_BY_LEVEL = "<LG_ResVehiclePurchaseCostsByWarehouseByLevel_V1>"
LG_RES_VEHICLE_MAINT_REPAIR_COSTS_BY_LEVEL = "<LG_ResVehicleMaintRepairCostsByLevel_V1>"
LG_RES_VEHICLE_MAINT_REPAIR_COSTS_BY_WAREHOUSE = "<LG_ResVehicleMaintRepairCostsByWarehouse_V1>"
LG_RES_VEHICLE_MAINT_REPAIR_COSTS_BY_WAREHOUSE_BY_LEVEL = "<LG_ResVehicleMaintRepairCostsByWarehouseByLevel_V1>"
LG_RES_VEHICLE_FUEL_COSTS_BY_LEVEL = "<LG_ResVehicleFuelCostsByLevel_V1>"
LG_RES_VEHICLE_FUEL_COSTS_BY_WAREHOUSE = "<LG_ResVehicleFuelCostsByWarehouse_V1>"
LG_RES_VEHICLE_FUEL_COSTS_BY_WAREHOUSE_BY_LEVEL = "<LG_ResVehicleFuelCostsByWarehouseByLevel_V1>"
# Volume analytics
LG_RES_NUM_WAREHOUSES_VISITED_BY_LEVEL = "<LG_ResNumWarehousesVisitedByLevel_V1>"
LG_RES_NUM_WAREHOUSES_VISITED_BY_WAREHOUSE = "<LG_ResNumWarehousesVisitedByWarehouse_V1>"
LG_RES_NUM_WAREHOUSES_VISITED_BY_WAREHOUSE_BY_LEVEL = "<LG_ResNumWarehousesVisitedByWarehouseByLevel_V1>"
LG_RES_NUM_VEHICLE_TRIPS_BY_LEVEL = "<LG_ResNumVehicleTripsByLevel_V1>"
LG_RES_NUM_VEHICLE_TRIPS_BY_WAREHOUSE = "<LG_ResNumVehicleTripsByWarehouse_V1>"
LG_RES_NUM_VEHICLE_TRIPS_BY_WAREHOUSE_BY_LEVEL = "<LG_ResNumVehicleTripsByWarehouseByLevel_V1>"
LG_RES_VEHICLE_DISTANCE_TRAVELED_BY_LEVEL = "<LG_ResVehicleDistanceTraveledByLevel_V1>"
LG_RES_VEHICLE_DISTANCE_TRAVELED_BY_WAREHOUSE = "<LG_ResVehicleDistanceTraveledByWarehouse_V1>"
LG_RES_VEHICLE_DISTANCE_TRAVELED_BY_WAREHOUSE_BY_LEVEL = "<LG_ResVehicleDistanceTraveledByWarehouseByLevel_V1>"
LG_RES_VEHICLE_UTILIZATION_VOLUME_BY_LEVEL = "<LG_ResVehicleUtilizationVolumeByLevel_V1>"
LG_RES_VEHICLE_UTILIZATION_VOLUME_BY_WAREHOUSE = "<LG_ResVehicleUtilizationVolumeByWarehouse_V1>"
LG_RES_VEHICLE_UTILIZATION_VOLUME_BY_WAREHOUSE_BY_LEVEL = "<LG_ResVehicleUtilizationVolumeByWarehouseByLevel_V1>"
LG_RES_VEHICLE_UTILIZATION_TIME_BY_LEVEL = "<LG_ResVehicleUtilizationTimeByLevel_V1>"
LG_RES_VEHICLE_UTILIZATION_TIME_BY_WAREHOUSE = "<LG_ResVehicleUtilizationTimeByWarehouse_V1>"
LG_RES_VEHICLE_UTILIZATION_TIME_BY_WAREHOUSE_BY_LEVEL = "<LG_ResVehicleUtilizationTimeByWarehouseByLevel_V1>"
LG_RES_TIME_NEEDED_PER_ROUND_TRIP_BY_LEVEL = "<LG_ResTimeNeededPerRoundTripByLevel_V1>"
LG_RES_TIME_NEEDED_PER_ROUND_TRIP_BY_WAREHOUSE = "<LG_ResTimeNeededPerRoundTripByWarehouse_V1>"
LG_RES_TIME_NEEDED_PER_ROUND_TRIP_BY_WAREHOUSE_BY_LEVEL = "<LG_ResTimeNeededPerRoundTripByWarehouseByLevel_V1>"
LG_RES_NUM_DAYS_OUT_ON_DELIVERY_BY_LEVEL = "<LG_ResNumDaysOutOnDeliveryByLevel_V1>"
LG_RES_NUM_DAYS_OUT_ON_DELIVERY_BY_WAREHOUSE = "<LG_ResNumDaysOutOnDeliveryByWarehouse_V1>"
LG_RES_NUM_DAYS_OUT_ON_DELIVERY_BY_WAREHOUSE_BY_LEVEL = "<LG_ResNumDaysOutOnDeliveryByWarehouseByLevel_V1>"


# Workers
LG_RES_NUM_WORKERS_BY_LEVEL = "<LG_ResNumWorkersByLevel_V1>"
LG_RES_NUM_WORKERS_BY_WAREHOUSE = "<LG_ResNumWorkersByWarehouse_V1>"
LG_RES_NUM_WORKERS_BY_WAREHOUSE_BY_LEVEL = "<LG_ResNumWorkersByWarehouseByLevel_V1>"
LG_RES_WORKER_SALARIES_BY_LEVEL = "<LG_ResWorkerSalariesByLevel_V1>"
LG_RES_WORKER_SALARIES_BY_WAREHOUSE = "<LG_ResWorkerSalariesByWarehouse_V1>"
LG_RES_WORKER_SALARIES_BY_WAREHOUSE_BY_LEVEL = "<LG_ResWorkerSalariesByWarehouseByLevel_V1>"
LG_RES_WORKER_BENEFITS_BY_LEVEL = "<LG_ResWorkerBenefitsByLevel_V1>"
LG_RES_WORKER_BENEFITS_BY_WAREHOUSE = "<LG_ResWorkerBenefitsByWarehouse_V1>"
LG_RES_WORKER_BENEFITS_BY_WAREHOUSE_BY_LEVEL = "<LG_ResWorkerBenefitsByWarehouseByLevel_V1>"

# Cold chain
LG_RES_NUM_COLD_CHAIN_AVAIL_BY_LEVEL = "<LG_ResNumColdChainAvailByLevel_V1>"
LG_RES_NUM_COLD_CHAIN_AVAIL_BY_WAREHOUSE = "<LG_ResNumColdChainAvailByWarehouse_V1>"
LG_RES_NUM_COLD_CHAIN_AVAIL_BY_WAREHOUSE_BY_LEVEL = "<LG_ResNumColdChainAvailByWarehouseByLevel_V1>"
LG_RES_NUM_COLD_CHAIN_PURCHASED_BY_LEVEL = "<LG_ResNumColdChainPurchasedByLevel_V1>"
LG_RES_NUM_COLD_CHAIN_PURCHASED_BY_WAREHOUSE = "<LG_ResNumColdChainPurchasedByWarehouse_V1>"
LG_RES_NUM_COLD_CHAIN_PURCHASED_BY_WAREHOUSE_BY_LEVEL = "<LG_ResNumColdChainPurchasedByWarehouseByLevel_V1>"
LG_RES_COLD_CHAIN_PURCHASE_COSTS_BY_LEVEL = "<LG_ResColdChainPurchaseCostsByLevel_V1>"
LG_RES_COLD_CHAIN_PURCHASE_COSTS_BY_WAREHOUSE = "<LG_ResColdChainPurchaseCostsByWarehouse_V1>"
LG_RES_COLD_CHAIN_PURCHASE_COSTS_BY_WAREHOUSE_BY_LEVEL = "<LG_ResColdChainPurchaseCostsByWarehouseByLevel_V1>"
LG_RES_COLD_CHAIN_MAINT_COSTS_BY_LEVEL = "<LG_ResColdChainMaintCostsByLevel_V1>"
LG_RES_COLD_CHAIN_MAINT_COSTS_BY_WAREHOUSE = "<LG_ResColdChainMaintCostsByWarehouse_V1>"
LG_RES_COLD_CHAIN_MAINT_COSTS_BY_WAREHOUSE_BY_LEVEL = "<LG_ResColdChainMaintCostsByWarehouseByLevel_V1>"

# Drugs and supplies
LG_RES_DRUG_SUPPLY_QUANTITIES_NEEDED = "<LG_ResDrugSupplyQuantitiesNeeded_V1>"
LG_RES_DRUG_SUPPLY_PURCHASE_COSTS = "<LG_ResDrugSupplyPurchaseCosts_V1>"
LG_RES_DRUG_SUPPLY_WASTAGE_COSTS = "<LG_ResDrugSupplyWastageCosts_V1>"
LG_RES_DRUG_SUPPLY_WASTAGE_QUANTITIES = "<LG_ResDrugSupplyWastageQuantities_V1>"
LG_RES_DRUG_SUPPLY_SAFETY_STOCK_NEEDED = "<LG_ResDrugSupplySafetyStockNeeded_V1>"
LG_RES_DRUG_SUPPLY_SAFETY_STOCK_AVAILABLE = "<LG_ResDrugSupplySafetyStockAvailable_V1>"
LG_RES_DRUG_SUPPLY_SAFETY_STOCK_NEED_PURCH = "<LG_ResDrugSupplySafetyStockNeedingToBePurchased_V1>"
LG_RES_DRUG_SUPPLY_SAFETY_STOCK_PURCHASE_COSTS = "<LG_ResDrugSupplySafetyStockPurchaseCosts_V1>"
LG_RES_DRUG_SUPPLY_WEIGHTS = "<LG_ResDrugSupplyWeights_V1>"
LG_RES_DRUG_SUPPLY_VOLUMES = "<LG_ResDrugSupplyVolumes_V1>"
LG_RES_DRUG_SUPPLY_WASTAGE_WEIGHTS = "<LG_ResDrugSupplyWastageWeights_V1>"
LG_RES_DRUG_SUPPLY_WASTAGE_VOLUMES = "<LG_ResDrugSupplyWastageVolumes_V1>"
LG_RES_DRUG_SUPPLY_TOTAL_COLD_CHAIN_VOLUMES = "<LG_ResDrugSupplyTotalColdChainVolumes_V1>"



from copy import deepcopy

from SpectrumCommon.Const.TB  import *



def replace_in_list(ls, value):
    for i,e in enumerate(ls):
        if type(e)==list:
            replace_in_list(ls[i], value)
        else:
            ls[i] = value

def PatientClincalAssessmentAlgUsed():
    


    IDs = [[TB_ClinicalAssessment, TB_ChestXRay],                       # PTB_HIVn_014             
           [TB_ClinicalAssessment, TB_ChestXRay, TB_ComputerAidedDtct], # PTB_HIVn_15p
           [TB_W3SS],                                                   # PTB_HIVp_09 
           [TB_4SymptomScreening, TB_C_ReactiveProtein],                # PTB_HIVp_1014
           [TB_4SymptomScreening, TB_C_ReactiveProtein],                # PTB_HIVp_15p
           [TB_ClinicalExtraPulm],           # ETB_HIVn_014
           [TB_ClinicalExtraPulm],           # ETB_HIVn_15p
           [TB_ClinicalExtraPulm],           # ETB_HIVp_09
           [TB_ClinicalExtraPulm],           # ETB_HIVp_1014
           [TB_ClinicalExtraPulm],           # ETB_HIVp_15p
    ]

    Perc_Receiving = deepcopy(IDs)
    replace_in_list(Perc_Receiving, 1.00)

    return (Perc_Receiving, IDs)






def HouseholdClincalAssessmentAlgUsed():
    

    IDs = [[TB_W4SS, TB_ChestXRay],                                     # TB_HH0_4             
           [TB_SingleSymptomScrn],                                     # TB_HH5_14
           [TB_ClinicalAssessment, TB_ChestXRay, TB_ComputerAidedDtct], # TB_HH15p
 
    ]

    Perc_Receiving = deepcopy(IDs)
    replace_in_list(Perc_Receiving, 1.00)

    return (Perc_Receiving, IDs)


def HouseholdPreventionMethodsUsed():
    

    IDs = [[TB_TuberculinSkinTest, TB_AntigenSkinTest, TB_IGRA],  # TB_HH0_4             
           [TB_TuberculinSkinTest, TB_AntigenSkinTest, TB_IGRA], # TB_HH5_14
           [TB_TuberculinSkinTest, TB_AntigenSkinTest, TB_IGRA], # TB_HH15p
 
    ]
    
    
    
    Perc_Receiving = deepcopy(IDs)
    replace_in_list(Perc_Receiving, 0.00)
    for hh in TB_HHList:
        Perc_Receiving[hh][0] = 0.65 # Tuberculin skin test (TST)	65%
        Perc_Receiving[hh][1] = 0.05 # TB antigen-based skin test (TBST) for diagnosis	5%
        Perc_Receiving[hh][2] = 0.30 # Interferon-gamma release assay (IGRA)	30%

    return (Perc_Receiving, IDs)


def ARTClincalAssessmentAlgUsed():
    

    IDs = [[TB_W3SS],         # TB_HH0_4             
           [TB_mWRDPrevGT10], # TB_HH5_14
           [TB_mWRDPrevGT10], # TB_HH15p
 
    ]

    Perc_Receiving = deepcopy(IDs)
    replace_in_list(Perc_Receiving, 1.00)

    return ([Perc_Receiving,Perc_Receiving] , [IDs, IDs])




def ARTPreventionMethodsUsed():
    child = [TB_TuberculinSkinTest, TB_AntigenSkinTest, TB_IGRA]
    adult = [TB_TuberculinSkinTest, TB_AntigenSkinTest, TB_IGRA]    
   
    IDs = [child, # TB_HH0_4             
           adult, # TB_HH5_14
           adult, # TB_HH15p
 
    ]

    Perc_Receiving = deepcopy(IDs)
    replace_in_list(Perc_Receiving, 0.00)
    for i,e in enumerate(Perc_Receiving):
        Perc_Receiving[i][0] = 0.65 # Tuberculin skin test (TST)	65%
        Perc_Receiving[i][1] = 0.05 # TB antigen-based skin test (TBST) for diagnosis	5%
        Perc_Receiving[i][2] = 0.30 # Interferon-gamma release assay (IGRA)	30%

    return ([Perc_Receiving, Perc_Receiving], [IDs, IDs])


def HRGClincalAssessmentAlgUsed():
    

    IDs = [[TB_ClinicalAssessment, TB_ChestXRay, TB_ComputerAidedDtct], # TB_HRG_Prisoners                     
           [TB_ClinicalAssessment, TB_ChestXRay, TB_ComputerAidedDtct], # TB_HRG_Miners           
           [TB_4SymptomScreening, TB_ChestXRay, TB_ComputerAidedDtct], # TB_HRG_SeekingCare      
           [TB_4SymptomScreening, TB_ChestXRay, TB_ComputerAidedDtct], # TB_HRG_StructuralRFs    
           [TB_4SymptomScreening, TB_ChestXRay, TB_ComputerAidedDtct], # TB_HRG_HighTBPrev       
           [TB_4SymptomScreening, TB_ChestXRay, TB_ComputerAidedDtct], # TB_HRG_UntreatedFibrosis
 
    ]

  

    Perc_Receiving = deepcopy(IDs)
    replace_in_list(Perc_Receiving, 1.00)

    return (Perc_Receiving, IDs)




def HRGPreventionMethodsUsed():
    

    IDs = [
           [TB_TuberculinSkinTest, TB_AntigenSkinTest, TB_IGRA], # TB_HRG_Prisoners                     
           [TB_TuberculinSkinTest, TB_AntigenSkinTest, TB_IGRA], # TB_HRG_Miners           
           [TB_TuberculinSkinTest, TB_AntigenSkinTest, TB_IGRA], # TB_HRG_SeekingCare      
           [TB_TuberculinSkinTest, TB_AntigenSkinTest, TB_IGRA], # TB_HRG_StructuralRFs    
           [TB_TuberculinSkinTest, TB_AntigenSkinTest, TB_IGRA], # TB_HRG_HighTBPrev       
           [TB_TuberculinSkinTest, TB_AntigenSkinTest, TB_IGRA], # TB_HRG_UntreatedFibrosis
 
    ]

    Perc_Receiving = deepcopy(IDs)
    replace_in_list(Perc_Receiving, 0.00)
    for i,e in enumerate(Perc_Receiving):
        Perc_Receiving[i][0] = 0.65 # Tuberculin skin test (TST)	65%
        Perc_Receiving[i][1] = 0.05 # TB antigen-based skin test (TBST) for diagnosis	5%
        Perc_Receiving[i][2] = 0.30 # Interferon-gamma release assay (IGRA)	30%

    return (Perc_Receiving, IDs)


